@@ -3,7 +3,7 @@ use Moo;
 use Sub::Install qw/install_sub/;
 use Carp qw(croak);
 
-our $VERSION = '0.97_3';
+our $VERSION = '0.191.0';
 
 has 'sth' => (
     is       => 'ro',
@@ -55,6 +55,16 @@ sub BUILD {
 }
 
 sub next {
+    my $self = shift;
+    return $self->object;
+}
+
+sub all {
+    my $self = shift;
+    return $self->objects;
+}
+
+sub object {
     my $self = shift;
     return if ( $self->_done );
 
@@ -67,14 +77,66 @@ sub next {
     return bless \@values, $self->class;
 }
 
-sub all {
+sub objects {
+    my $self = shift;
+    my $all  = $self->arrays;
+
+    my $class = $self->class;
+
+    foreach ( 0 .. $#{$all} ) {
+        bless $all->[$_], $class;
+    }
+
+    $self->finish;
+    return @$all if wantarray;
+    return $all;
+}
+
+sub hash {
+    my $self = shift;
+    return if ( $self->_done );
+
+    my $ref = $self->sth->fetchrow_hashref('NAME_lc');
+
+    if ( !$ref ) {
+        $self->finish;
+        return;
+    }
+    return $ref;
+}
+
+sub hashes {
     my $self = shift;
     my @all;
-    while ( my @values = $self->sth->fetchrow_array ) {
-        push( @all, bless \@values, $self->class );
+
+    while ( my $ref = $self->sth->fetchrow_hashref('NAME_lc') ) {
+        push( @all, $ref );
     }
     $self->finish;
-    return @all;
+    return @all if wantarray;
+    return \@all;
+}
+
+sub array {
+    my $self = shift;
+    return if ( $self->_done );
+
+    my $ref = $self->sth->fetchrow_arrayref;
+
+    if ( !$ref ) {
+        $self->finish;
+        return;
+    }
+    return $ref;
+}
+
+sub arrays {
+    my $self = shift;
+
+    my $all = $self->sth->fetchall_arrayref;
+    $self->finish;
+    return @$all if wantarray;
+    return $all;
 }
 
 sub finish {
