@@ -2,19 +2,12 @@ package SQL::DB::Schema;
 use strict;
 use warnings;
 use Moo;
+use Log::Any qw/$log/;
 use Carp qw/confess/;
-use SQL::DB::Expr;
+use SQL::DB::Expr qw/_bval/;
 use Sub::Install qw/install_sub/;
-use Sub::Exporter -setup => {
-    exports => ['get_schema'],
-    groups  => {
-        all     => ['get_schema'],
-        default => [],
-    },
-};
 
-our $VERSION = '0.97_3';
-my %schema;
+our $VERSION = '0.191.0';
 
 # Ordinals for DBI->column_info() results
 use constant {
@@ -41,15 +34,17 @@ use constant {
 # Object definition
 
 has 'name' => (
-    is       => 'rw',
+    is       => 'ro',
     required => 1,
 );
 
-has '_package_root' => (
-    is       => 'rw',
-    init_arg => undef,
+has 'package_root' => (
+    is       => 'ro',
+    required => 1,
 );
 
+has 'load' => ( is => 'ro', );
+
 has '_tables' => (
     is       => 'ro',
     init_arg => undef,
@@ -58,19 +53,35 @@ has '_tables' => (
 
 sub _getglob { no strict 'refs'; \*{ $_[0] } }
 
+around BUILDARGS => sub {
+    my $orig  = shift;
+    my $class = shift;
+    my %args  = @_;
+
+    ( $args{package_root} = $args{name} ) =~ tr/a-zA-Z0-9/_/cs;
+    $args{package_root} = __PACKAGE__ . '::' . $args{package_root};
+
+    return $class->$orig(%args);
+};
+
 sub BUILD {
     my $self = shift;
-    ( my $name = $self->name ) =~ tr/a-zA-Z0-9/_/cs;
-    $self->name($name);
-    $self->_package_root( __PACKAGE__ . '::' . $name );
-    $schema{$name} = $self;
+
+    if ( $self->load ) {
+        my $class = $self->name;
+        confess $@ unless eval "require $class;";
+
+        $self->define( $class->definition );
+        $class->clear;
+        $log->debug( 'Loaded schema', $class );
+    }
 }
 
 sub define {
     my $self = shift;
     my $data = shift;
 
-    my $package_root = $self->_package_root;
+    my $package_root = $self->package_root;
     my $tables       = $self->_tables;
 
     foreach my $colref (@$data) {
@@ -90,7 +101,7 @@ sub define {
                 {
                     code => sub {
                         my $table_expr = shift;
-                        return '*';
+                        return $table_expr . '.*';
                     },
                     into => $urow,
                     as   => '_columns',
@@ -100,7 +111,8 @@ sub define {
                 {
                     code => sub {
                         my $table_expr = shift;
-                        return '*';
+                        return SQL::DB::Expr->new(
+                            _txt => [ $table_expr->_alias . '.*' ] );
                     },
                     into => $srow,
                     as   => '_columns',
@@ -109,7 +121,14 @@ sub define {
         }
         $tables->{$table}++;
 
-        my $col  = $colref->[COLUMN_NAME];
+        my $col = lc $colref->[COLUMN_NAME];
+
+        if ( $col eq 'new' ) {
+            confess "Column name 'new' (table/view '$table') clashes with "
+              . __PACKAGE__ . '!!!';
+        }
+
+        use bytes;
         my $type = lc $colref->[TYPE_NAME];
 
         install_sub(
@@ -117,8 +136,8 @@ sub define {
                 code => sub {
                     my $table_expr = shift;
                     SQL::DB::Expr->new(
-                        _txt   => $table_expr->_alias . '.' . $col,
-                        _btype => $type,
+                        _txt  => [ $table_expr->_alias . '.' . $col ],
+                        _type => $type,
                     );
                 },
                 into => $srow,
@@ -134,15 +153,14 @@ sub define {
                     if (@_) {
                         my $val = shift;
                         return SQL::DB::Expr->new(
-                            _txt     => $col . ' = ?',
-                            _btype   => $type,
-                            _bvalues => [$val],
+                            _txt  => [ $col . ' = ', _bval( $val, $type ) ],
+                            _type => $type,
                         );
                     }
 
                     return SQL::DB::Expr->new(
-                        _txt   => $col,
-                        _btype => $type,
+                        _txt  => [$col],
+                        _type => $type,
                     );
                 },
                 into => $urow,
@@ -160,14 +178,30 @@ sub not_known {
     return grep { !exists $tables->{$_} } @_;
 }
 
+sub irow {
+    my $self = shift;
+
+    my @ret;
+    foreach my $name (@_) {
+        if ( !exists $self->_tables->{$name} ) {
+            confess "Table not defined in schema: $name";
+        }
+        push( @ret, sub { $name . '(' . join( ',', @_ ) . ')' } );
+        return $ret[0] unless (wantarray);
+    }
+    return @ret;
+}
+
 sub srow {
     my $self = shift;
 
     my @ret;
     foreach my $name (@_) {
-        my $class = $self->_package_root . '::Srow::' . $name;
-        my $srow = eval { $class->new( _txt => $name, _alias => $name ) };
-        confess "$@\nTable not defined in schema?: $name" if $@;
+        if ( !exists $self->_tables->{$name} ) {
+            confess "Table not defined in schema: $name";
+        }
+        my $class = $self->package_root . '::Srow::' . $name;
+        my $srow = $class->new( _txt => [$name], _alias => $name );
         return $srow unless (wantarray);
         push( @ret, $srow );
     }
@@ -179,23 +213,17 @@ sub urow {
 
     my @ret;
     foreach my $name (@_) {
-        my $class = $self->_package_root . '::Urow::' . $name;
-        my $urow = eval { $class->new( _txt => $name ) };
-        confess "$@\nTable not defined in schema?: $name" if $@;
+        if ( !exists $self->_tables->{$name} ) {
+            confess "Table not defined in schema: $name";
+        }
+        my $class = $self->package_root . '::Urow::' . $name;
+        my $urow = $class->new( _txt => [$name] );
         return $urow unless (wantarray);
         push( @ret, $urow );
     }
     return @ret;
 }
 
-# Class functions
-
-sub get_schema {
-    my $name = shift;
-    return $schema{$name} if ( exists $schema{$name} );
-    return;
-}
-
 1;
 
 # vim: set tabstop=4 expandtab:
